@@ -1,12 +1,18 @@
+# -*- coding: utf-8 -*-
+from __future__ import absolute_import, division, print_function, with_statement
+
 import os
 import shutil
 import unittest
 import kaptan
-from ..config import expand_config, trickledown_config
+from .. import config, exc
 from ..util import tmux
 
+from .. import log
+import logging
 
-TMUXWRAPPER_DIR = os.path.join(os.path.dirname(__file__), '.tmuxp')
+logger = logging.getLogger(__name__)
+TMUXP_DIR = os.path.join(os.path.dirname(__file__), '.tmuxp')
 
 sampleconfigdict = {
     'session_name': 'sampleconfig',
@@ -33,24 +39,21 @@ sampleconfigdict = {
 }
 
 
-class ConfigImportExportTestCase(unittest.TestCase):
+class ImportExportTest(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
-        # run parent
-        # setUpClass
-        if not os.path.exists(TMUXWRAPPER_DIR):
-            os.makedirs(
-                TMUXWRAPPER_DIR)
-            # super(ConfigTest, cls).setUpClass()
+        if not os.path.exists(TMUXP_DIR):
+            os.makedirs(TMUXP_DIR)
 
     def test_export_json(self):
-        json_config_file = os.path.join(TMUXWRAPPER_DIR, 'config.json')
+        json_config_file = os.path.join(TMUXP_DIR, 'config.json')
 
-        config = kaptan.Kaptan()
-        config.import_config(sampleconfigdict)
+        configparser = kaptan.Kaptan()
+        sampleconfig = config.inline(sampleconfigdict)
+        configparser.import_config(sampleconfigdict)
 
-        json_config_data = config.export('json', indent=2)
+        json_config_data = configparser.export('json', indent=2)
 
         buf = open(json_config_file, 'w')
         buf.write(json_config_data)
@@ -61,12 +64,13 @@ class ConfigImportExportTestCase(unittest.TestCase):
         self.assertDictEqual(sampleconfigdict, new_config_data)
 
     def test_export_yaml(self):
-        yaml_config_file = os.path.join(TMUXWRAPPER_DIR, 'config.yaml')
+        yaml_config_file = os.path.join(TMUXP_DIR, 'config.yaml')
 
-        config = kaptan.Kaptan()
-        config.import_config(sampleconfigdict)
+        configparser = kaptan.Kaptan()
+        sampleconfig = config.inline(sampleconfigdict)
+        configparser.import_config(sampleconfig)
 
-        yaml_config_data = config.export('yaml', indent=2)
+        yaml_config_data = configparser.export('yaml', indent=2, default_flow_style=False)
 
         buf = open(yaml_config_file, 'w')
         buf.write(yaml_config_data)
@@ -79,41 +83,42 @@ class ConfigImportExportTestCase(unittest.TestCase):
     def test_scan_config(self):
         configs = []
 
-        garbage_file = os.path.join(TMUXWRAPPER_DIR, 'config.psd')
+        garbage_file = os.path.join(TMUXP_DIR, 'config.psd')
         buf = open(garbage_file, 'w')
         buf.write('wat')
         buf.close()
 
-        if os.path.exists(TMUXWRAPPER_DIR):
-            for r, d, f in os.walk(TMUXWRAPPER_DIR):
+        if os.path.exists(TMUXP_DIR):
+            for r, d, f in os.walk(TMUXP_DIR):
                 for filela in (x for x in f if x.endswith(('.json', '.ini', 'yaml'))):
                     configs.append(os.path.join(
-                        TMUXWRAPPER_DIR, filela))
+                        TMUXP_DIR, filela))
 
         files = 0
-        if os.path.exists(os.path.join(TMUXWRAPPER_DIR, 'config.json')):
+        if os.path.exists(os.path.join(TMUXP_DIR, 'config.json')):
             files += 1
             self.assertIn(os.path.join(
-                TMUXWRAPPER_DIR, 'config.json'), configs)
+                TMUXP_DIR, 'config.json'), configs)
 
-        if os.path.exists(os.path.join(TMUXWRAPPER_DIR, 'config.yaml')):
+        if os.path.exists(os.path.join(TMUXP_DIR, 'config.yaml')):
             files += 1
             self.assertIn(os.path.join(
-                TMUXWRAPPER_DIR, 'config.yaml'), configs)
+                TMUXP_DIR, 'config.yaml'), configs)
 
-        if os.path.exists(os.path.join(TMUXWRAPPER_DIR, 'config.ini')):
+        if os.path.exists(os.path.join(TMUXP_DIR, 'config.ini')):
             files += 1
-            self.assertIn(os.path.join(TMUXWRAPPER_DIR, 'config.ini'), configs)
+            self.assertIn(os.path.join(TMUXP_DIR, 'config.ini'), configs)
 
         self.assertEqual(len(configs), files)
 
     @classmethod
     def tearDownClass(cls):
-        if os.path.isdir(TMUXWRAPPER_DIR):
-            shutil.rmtree(TMUXWRAPPER_DIR)
+        #if os.path.isdir(TMUXP_DIR):
+        #    shutil.rmtree(TMUXP_DIR)
+        logging.debug('wiped %s' % TMUXP_DIR)
 
 
-class ConfigExpandTestCase(unittest.TestCase):
+class ExpandTest(unittest.TestCase):
 
     '''
     assumes the configuration has been imported into a python dict correctly.
@@ -143,12 +148,55 @@ class ConfigExpandTestCase(unittest.TestCase):
             {
                 'automatic_rename': True,
                 'panes': [
-                    {'shell_command': 'htop'}
+                    {'shell_command': 'htop'},
+                    'vim',
                 ]
             }]
     }
 
     after_config = {
+        'session_name': 'sampleconfig',
+        'start_directory': '~',
+        'windows': [{
+            'shell_command': ['top'],
+            'window_name': 'editor',
+            'panes': [
+                {
+                    'start_directory': '~', 'shell_command': ['vim'],
+                    },  {
+                    'shell_command': ['cowsay "hey"']
+                },
+            ],
+            'layout': 'main-verticle'},
+            {
+                'window_name': 'logging',
+                'panes': [
+                    {'shell_command': ['tail -F /var/log/syslog'],
+                     'start_directory':'/var/log'}
+                ]
+            },
+            {
+                'automatic_rename': True,
+                'panes': [
+                    {'shell_command': ['htop']},
+                    {'shell_command': ['vim']}
+                ]
+            }]
+    }
+
+    def test_config(self):
+        '''
+        expands shell commands from string to list
+        '''
+        test_config = config.expand(self.before_config)
+        self.assertDictEqual(test_config, self.after_config)
+
+
+class InlineTest(unittest.TestCase):
+    '''tests for :meth:`config.inline()`.
+    '''
+
+    before_config = {
         'session_name': 'sampleconfig',
         'start_directory': '~',
         'windows': [{
@@ -177,15 +225,45 @@ class ConfigExpandTestCase(unittest.TestCase):
             }]
     }
 
-    def test_expand_config(self):
+    after_config = {
+        'session_name': 'sampleconfig',
+        'start_directory': '~',
+        'windows': [{
+            'shell_command': 'top',
+            'window_name': 'editor',
+            'panes': [
+                {
+                    'start_directory': '~', 'shell_command': 'vim',
+                    },  {
+                    'shell_command': 'cowsay "hey"'
+                },
+            ],
+            'layout': 'main-verticle'},
+            {
+                'window_name': 'logging',
+                'panes': [
+                    {'shell_command': 'tail -F /var/log/syslog',
+                     'start_directory':'/var/log'}
+                ]
+            },
+            {
+                'automatic_rename': True,
+                'panes': [
+                    {'shell_command': 'htop'}
+                ]
+            }]
+    }
+
+    def test_config(self):
         '''
-        expands shell commands from string to list
+        config.inline() inlines shell commands from list to string where applicable
         '''
-        config = expand_config(self.before_config)
-        self.assertDictEqual(config, self.after_config)
+        self.maxDiff = None
+        test_config = config.inline(self.before_config)
+        self.assertDictEqual(test_config, self.after_config)
 
 
-class ConfigInheritanceStartCommandTestCase(unittest.TestCase):
+class InheritanceTest(unittest.TestCase):
 
     '''
     test config inheritance for the nested 'start_command'
@@ -290,7 +368,7 @@ class ConfigInheritanceStartCommandTestCase(unittest.TestCase):
         self.assertDictEqual(config, self.config_after)
 
 
-class ConfigShellCommandBefore(unittest.TestCase):
+class ShellCommandBeforeTest(unittest.TestCase):
 
     '''
     test config inheritance for the nested 'start_command'
@@ -422,14 +500,95 @@ class ConfigShellCommandBefore(unittest.TestCase):
     }
 
     def test_shell_command_before(self):
-        config = self.config_unexpanded
-        config = expand_config(config)
+        test_config = self.config_unexpanded
+        test_config = config.expand(test_config)
 
-        self.assertDictEqual(config, self.config_expanded)
+        self.assertDictEqual(test_config, self.config_expanded)
 
-        config = trickledown_config(config)
+        test_config = config.trickle(test_config)
         self.maxDiff = None
-        self.assertDictEqual(config, self.config_after)
+        self.assertDictEqual(test_config, self.config_after)
+
+
+class ConfigConsistency(unittest.TestCase):
+
+    delete_this = '''
+    session_name: sampleconfig
+    start_directory: '~'
+    windows:
+    - layout: main-vertical
+    panes:
+    - shell_command:
+        - vim
+        start_directory: '~'
+    - shell_command:
+        - cowsay "hey"
+    window_name: editor
+    - panes:
+    - shell_command:
+        - tail -F /var/log/syslog
+        start_directory: /var/log
+    window_name: logging
+    - automatic_rename: true
+    panes:
+    - shell_command:
+        - htop
+    '''
+
+    def test_no_session_name(self):
+        yaml_config = '''
+        - window_name: editor
+          panes:
+          shell_command:
+            - tail -F /var/log/syslog
+          start_directory: /var/log
+        - window_name: logging
+          automatic_rename: true
+          panes:
+          - shell_command:
+            - htop
+        '''
+
+        sconfig = kaptan.Kaptan(handler='yaml')
+        sconfig = sconfig.import_config(yaml_config).get()
+
+        with self.assertRaisesRegexp(exc.ConfigError, 'requires "session_name"'):
+            config.check_consistency(sconfig)
+
+    def test_no_windows(self):
+        yaml_config = '''
+        session_name: test session
+        '''
+
+        sconfig = kaptan.Kaptan(handler='yaml')
+        sconfig = sconfig.import_config(yaml_config).get()
+
+        with self.assertRaisesRegexp(exc.ConfigError, 'list of "windows"'):
+            config.check_consistency(sconfig)
+
+    def test_no_window_name(self):
+        yaml_config = '''
+        session_name: test session
+        windows:
+        - window_name: editor
+          panes:
+          shell_command:
+            - tail -F /var/log/syslog
+          start_directory: /var/log
+        - automatic_rename: true
+          panes:
+          - shell_command:
+            - htop
+        '''
+
+        sconfig = kaptan.Kaptan(handler='yaml')
+        sconfig = sconfig.import_config(yaml_config).get()
+
+        with self.assertRaisesRegexp(exc.ConfigError, 'missing "window_name"'):
+            config.check_consistency(sconfig)
+
+
+
 
 if __name__ == '__main__':
     unittest.main()
